<#
    .SYNOPSIS
        SplitXCI.
 
    .DESCRIPTION
        Split XCI into part files so that NX file system read them on FAT32 as a file.
#>
 
$scriptPath = $MyInvocation.MyCommand.Path
$scriptDirectoryPath = [System.IO.Path]::GetDirectoryName($scriptPath)
 
$UNIT_SIZE = 4 * 1024 * 1024 * 1024 - 64 * 1024
$BUFFER_SIZE = 8 * 1024 * 1024
$BLOCK_SIZE = 32 * 1024
 
$global:progress = [long]0
$global:totalSize = [long]0
 
function Write-Progress
{
    $tmp = [System.String]::Format("{0, 3} %", [int] ([double] $global:progress / $global:totalSize * 100))
	Clear-Host
    [System.Console]::Write("Creating split xci files:{0} complete. Please wait!", $tmp)
    return 0
}
 
function Copy-Stream ([System.IO.Stream] $src, [System.IO.Stream] $dst, [long] $size, [Byte[]] $buffer)
{
    $restSize = $size
    while ($restSize -gt 0)
    {
        $copySize = [System.Math]::Min($restSize, $buffer.Length)
        $read = $src.Read($buffer, 0, $copySize)
        if ($read -ne $copySize)
        {
            return $read
        }
        $dst.Write($buffer, 0, $copySize)
        $restSize -= [long]$copySize
        $global:progress += [long]$copySize
 
        Write-Progress | Out-Null
    }
    return $size
}
 
function Get-Index([long] $totalSize)
{
    $lastIndex = 0
    $restSize = $totalSize
 
    while ($restSize -gt 0)
    {
        $copySize = [System.Math]::Min($restSize, $UNIT_SIZE)
        $lastIndex += 1
        $restSize -= $copySize
    }
 
    return $lastIndex
}
 
foreach ($arg in $args)
{
    $fileInfo = New-Object System.IO.FileInfo($arg)
    if ($fileInfo.Length -lt $UNIT_SIZE)
    {
        [System.Console]::WriteLine("{0} is not need to be divided because its size is less than 0x{1:x} bytes.", $fileInfo.Basename, $UNIT_SIZE)
        continue
    }
    $outputDir = $scriptDirectoryPath + "\" + $fileInfo.Basename
	$xciname = $fileInfo.Basename
    [System.Console]::WriteLine("source path: {0}, bytes: {1}", $outputDir, $fileInfo.Length)
    [System.IO.Directory]::CreateDirectory($outputDir) | Out-Null
 
    # アーカイブ属性を付与
    $dirInfo = New-Object System.IO.DirectoryInfo($outputDir)
    $dirInfo.Attributes = $dirInfo.Attributes -bor [System.IO.FileAttributes]::Archive
 
    $restSize = $fileInfo.Length
    $global:totalSize = $fileInfo.Length
 
    $index = 0
    $lastIndex = Get-Index $restSize
 
    $source = New-Object System.IO.FileStream($arg, [System.IO.FileMode]::Open, [System.IO.FileAccess]::Read, [System.IO.FileShare]::Read, $BLOCK_SIZE, [System.IO.FileOptions]::SequentialScan)
    if ($source -eq $null)
    {
        [System.Console]::WriteLine("Failed to open {0}.", $arg)
        exit 1
    }
 
    try
    {
        $buffer = New-Object byte[] $BUFFER_SIZE
        $copied = [long]0
 
        while ($restSize -gt 0)
        {
            $copySize = [System.Math]::Min($restSize, $UNIT_SIZE)
            $targetPath = [System.String]::Format("{0}\{1:d2}", $outputDir, $index)
 
            [System.Console]::Write("creating {0} ({1}/{2})... ", $targetPath, $index + 1, $lastIndex)
            Write-Progress | Out-Null
 
            $target = New-Object System.IO.FileStream($targetPath, [System.IO.FileMode]::Create, [System.IO.FileAccess]::Write, [System.IO.FileShare]::None, $BLOCK_SIZE, [System.IO.FileOptions]::SequentialScan)
            if ($target -eq $null)
            {
                [System.Console]::WriteLine("Failed to create {0}.", $targetPath)
                exit 1
            }
            try
            {
                $ret = Copy-Stream $source $target $copySize $buffer
                if ($ret -ne $copySize)
                {
                    [System.Console]::WriteLine("Failed to copy from {0} to {1}. {2}", $arg, $targetPath, $ret)
                    exit 1
                }
            }
            finally
            {
                $target.Dispose()
            }
 
            [System.Console]::WriteLine("done.")
 
            $restSize -= $copySize
            $copied += [long]$copySize
            $index += 1
        }
    }
    finally
    {
        $source.Dispose()
    }
	
	if(Test-Path -path $outputDir\00)
	{
    Rename-Item $outputDir\00 $outputDir\$xciname.xc0
	}
	
	if(Test-Path -path $outputDir\01)
	{
    Rename-Item $outputDir\01 $outputDir\$xciname.xc1
	}
	
	if(Test-Path -path $outputDir\02)
	{
    Rename-Item $outputDir\02 $outputDir\$xciname.xc2
	}
	
	if(Test-Path -path $outputDir\03)
	{
    Rename-Item $outputDir\03 $outputDir\$xciname.xc3
	}
	
	if(Test-Path -path $outputDir\04)
	{
    Rename-Item $outputDir\04 $outputDir\$xciname.xc4
	}
	
	if(Test-Path -path $outputDir\05)
	{
    Rename-Item $outputDir\05 $outputDir\$xciname.xc5
	}
	
	if(Test-Path -path $outputDir\06)
	{
    Rename-Item $outputDir\06 $outputDir\$xciname.xc6
	}
	
	if(Test-Path -path $outputDir\07)
	{
    Rename-Item $outputDir\07 $outputDir\$xciname.xc7
	}
	
	if(Test-Path -path $outputDir\08)
	{
    Rename-Item $outputDir\08 $outputDir\$xciname.xc8
	}
	
    [System.Console]::WriteLine("XCI split completed. Put created files in the sdcard:/sxos/xci directory")
}
 
exit 0